package it.macke.is;

import java.io.IOException;
import java.lang.reflect.Field;

import com.wm.app.b2b.client.Context;
import com.wm.app.b2b.client.ServiceException;
import com.wm.data.IData;
import com.wm.data.IDataCursor;
import com.wm.data.IDataFactory;
import com.wm.data.IDataUtil;

public abstract class ServiceCall<TInput extends ServiceInput, TOutput extends ServiceOutput>
{
	public static final String HOST_DEVELOPMENT = "server:5555";

	protected final String username = "administrator";
	protected final String password = "password";

	protected String host;
	protected String servicePath;
	protected String serviceName;

	protected ServiceCall(String host, String servicePath, String serviceName)
	{
		this.host = host;
		this.servicePath = servicePath;
		this.serviceName = serviceName;
	}

	protected Context connectToServer() throws ServiceException
	{
		Context context = new Context();
		context.connect(host, username, password);
		return context;
	}

	protected IData invokeService(IData input) throws IOException, ServiceException
	{
		Context context = connectToServer();
		return context.invoke(servicePath, serviceName, input);
	}

	public TOutput call(TInput params) throws Exception
	{
		IData input = createPipelineFromInput(params);
		IData pipeline = invokeService(input);
		return createOutputFromPipeline(pipeline);
	}

	protected IData createPipelineFromInput(ServiceInput input)
	{
		IData pipeline = IDataFactory.create();
		IDataCursor idc = pipeline.getCursor();

		Class<?> c = input.getClass();
		for (Field publicField : c.getFields())
		{
			try
			{
				String fieldName = FieldNameConverter.convertToPipelineName(publicField.getName());
				Object fieldValue = FieldConverter.convertToPipeline(publicField.getType().getName(), publicField.get(input));
				IDataUtil.put(idc, fieldName, fieldValue);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		idc.destroy();

		return pipeline;
	}

	protected TOutput createOutputFromPipeline(IData pipeline)
	{
		IDataCursor idc = pipeline.getCursor();
		TOutput output = createOutput();

		Class<?> c = output.getClass();
		for (Field publicField : c.getFields())
		{
			try
			{
				String fieldName = FieldNameConverter.convertToFieldName(publicField.getName());
				Object fieldValue = PipelineConverter.convertToField(publicField.getType().getName(), IDataUtil.get(idc, fieldName));
				publicField.set(output, fieldValue);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		idc.destroy();

		return output;
	}

	protected abstract TOutput createOutput();
}