package it.macke.is.services;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;
import it.macke.is.ServiceCall;
import it.macke.is.common.ErrorInfo;

import org.junit.Before;
import org.junit.Test;

public class MapNaturalErrorShould
{
	protected MapNaturalError sut;
	protected MapNaturalError.Input input;

	@Before
	public void setup()
	{
		sut = new MapNaturalError(ServiceCall.HOST_DEVELOPMENT);
		input = sut.createInput();
		input.errorFlag = "false";
		input.errorCode = "0";
		input.errorMessage = "";
		input.ERROR_INFO = null;
	}

	private void assertErrorWasReturned(ErrorInfo error, String errorCode, String errorMessage, String errorSource)
	{
		assertThat(error.errorFlag, is("true"));
		assertThat(error.errors.length, is(1));
		assertThat(error.errors[0].errorCode, is(errorCode));
		assertThat(error.errors[0].errorMessage, is(errorMessage));
		assertThat(error.errors[0].errorSource, is(errorSource));
	}

	@Test
	public void returnNoErrorIfNoErrorWasGiven() throws Exception
	{
		MapNaturalError.Output output = sut.call(input);
		assertThat(output.error.errorFlag, is("false"));
		assertThat(output.error.errors.length, is(0));
	}

	@Test
	public void returnErrorIfErrorFlagIsTrue() throws Exception
	{
		input.errorFlag = "true";
		MapNaturalError.Output output = sut.call(input);
		assertErrorWasReturned(output.error, "1", "Error flag was set, but no error code or error message was given.", "Integration Server");
	}
}
